@@ -8,10 +8,10 @@
 ///
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Last changed  : $Date: 2009-05-17 19:48:30 +0300 (Sun, 17 May 2009) $
+// Last changed  : $Date: 2014-04-06 15:57:21 +0000 (Sun, 06 Apr 2014) $
 // File revision : $Revision: 4 $
 //
-// $Id: RunParameters.cpp 72 2009-05-17 16:48:30Z oparviai $
+// $Id: RunParameters.cpp 195 2014-04-06 15:57:21Z oparviai $
 //
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -36,7 +36,6 @@
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#include <stdexcept>
 #include <string>
 #include <stdlib.h>
 
@@ -120,7 +119,7 @@ RunParameters::RunParameters(const int nParams, const char * const paramStr[])
         }
         string msg = whatText;
         msg += usage;
-        throw runtime_error(msg.c_str());
+        ST_THROW_RT_ERROR(msg.c_str());
     }
 
     inFileName = NULL;
@@ -131,8 +130,8 @@ RunParameters::RunParameters(const int nParams, const char * const paramStr[])
     quick = 0;
     noAntiAlias = 0;
     goalBPM = 0;
-    speech = FALSE;
-    detectBPM = FALSE;
+    speech = false;
+    detectBPM = false;
 
     // Get input & output file names
     inFileName = (char*)paramStr[1];
@@ -200,14 +199,14 @@ void RunParameters::throwIllegalParamExp(const string &str) const
     msg += str;
     msg += "\".\n\n";
     msg += usage;
-    throw runtime_error(msg.c_str());
+    ST_THROW_RT_ERROR(msg.c_str());
 }
 
 
 
 void RunParameters::throwLicense() const
 {
-    throw runtime_error(licenseText);
+    ST_THROW_RT_ERROR(licenseText);
 }
 
 
@@ -263,7 +262,7 @@ void RunParameters::parseSwitchParam(const string &str)
 
         case 'b' :
             // switch '-bpm=xx'
-            detectBPM = TRUE;
+            detectBPM = true;
             try
             {
                 goalBPM = parseSwitchValue(str);
@@ -292,7 +291,7 @@ void RunParameters::parseSwitchParam(const string &str)
 
         case 's' :
             // switch '-speech'
-            speech = TRUE;
+            speech = true;
             break;
 
         default:
